;; File description:
;; Functions to filter the CHILDES files of utterances
;; by mother and by child.
;; 
;; CHANGE RECORD
;; Ini  Date      Change summary & functions added, deleted, or altered.
;; ---  --------  ------------------------------------------------------
;; FRG  ??        Initial program.
;;                ------------------------------------------------------
;; GJ   29/10/98  Duplicated filter, remove-all-superfluous-characters,
;;                and print-this-line, for filtering some files of
;;                Caroline's. The duplicated functions are suffixed -cfh.
;;                Changed name of treat-pause to treat-pause-separate-utterances,
;;                and added a function treat-pause-same-utterance, so that a pause
;;                can either be treated as a separate utterance or the same
;;                utterance. Also added remove-all-repeats, though its
;;                functionality is not in place yet.
;;                A new load is put in which loads the general-functions
;;                file. The change-all-quotes function is used from this.
;;                ------------------------------------------------------
;; GJ   30/10/98  There's no need to put any list brackets around text
;;                because each line of text can be read in as a 
;;                string and converted to a list.
;;                ------------------------------------------------------
;; GJ   30/10/98  In remove-all-superfluous-characters, no longer 
;;                remove quotes (they get changed to @ signs), and
;;                for -' I just remove the -. Also double spaces
;;                are kept in now (there was no point in removing
;;                them, plus words separated by two spaces would
;;                become joined).
;;                ------------------------------------------------------
;; GJ   02/11/98  Remove 0's from the input file. Changed 
;;                remove-all-superfluous-characters.
;;                ------------------------------------------------------
;; GJ   09/11/98  Remove-sub-string now removes ALL occurrences of a
;;                substring rather than just the first.
;;                ------------------------------------------------------
;; GJ   09/11/98  Added function cater-for-verb-endings to deal with
;;                the type of coding files we're using as input. This
;;                function deals with -' and ' characters (it is
;;                called before removing superfluous characters in
;;                case we put in general removes for -' and ' characters
;;                as a general sweep).
;;                Also don't remove + signs (these link two nouns as
;;                one word e.g. santa+claus) - keep them in and
;;                when the file eventually gets converted to
;;                phonemes, the phonemic representation of each
;;                noun will be retrieved.
;;                ------------------------------------------------------
;; GJ   12/11/98  Updated all comments so that they are in-line with 
;;                what the code actually does. Removed unnecessary
;;                comments and code.
;;                ------------------------------------------------------
;; GJ   18/11/98  For commas, remove everything after (and including)
;;                the comma. This is done by the
;;                remove-everything-after-char function. Also, a
;;                count is kept of the number of input lines, and the
;;                number of lines containing a comma, so that
;;                a percentage can be output into a file.
;;                ------------------------------------------------------
;; GJ   02/12/98  For repeat marks ([/] and [//]), remove the previous
;;                word if applicable (if its a phrase it will be in
;;                angled brackets, which are removed by other functions).
;;                If the previous item is a pause the word prior to
;;                the pause needs to be removed.
;;                ------------------------------------------------------
;; GJ   03/12/98  Added remove-nonsense-words, to remove utterances
;;                oh, um, and er.
;;                ------------------------------------------------------
;; GJ   04/12/98  Some utterances in the Childes file take up more than
;;                one line - using read-line would read the utterance as
;;                two separate utterances. The files have to be pre-filtered
;;                to have all utterances using up only one line.
;;                The remove-imbedded-return-chars function does this (at the
;;                same time it removes tab characters as well).
;;                ------------------------------------------------------
;; GJ   22/01/99  Remove lines containing a vertical bar (|) and remove
;;                the characters +/ and +// from any lines.
;;                ------------------------------------------------------
;; GJ   27/02/99  Added a check of lisp-implementation-version to determine
;;                whether to load fasl files or pfsl (vsn 4+) files.
;;                ------------------------------------------------------
;; GJ   27/02/99  Remove slash characters (/) from lines.
;;                ------------------------------------------------------
;; GJ   03/03/99  All lexicon related files will be loaded from
;;                lexicon-loader.lisp now, so the initial setting of
;;                *hard-drive*, and loads for logical-dir and 
;;                general-functions, can be commented out. Also
;;                *project-directory* should be used for filepaths.
;;                ------------------------------------------------------
;; GJ   11/03/99  Before writing to output files, delete the file if it
;;                already exists. This is because overwrite in Lisp
;;                leaves the tail end of the previous file if the
;;                overwritten information is shorter.
;; ======== END OF CHANGE RECORD ========


;(cond
;  ((member "4" (mapcar #'string (coerce (lisp-implementation-version) 'list))
;           :test #'equal)
;    (load (concatenate 'string  *hard-drive*
;                                ":logical-dir-compatibility.pfsl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.pfsl")))
;  (t
;    (load (concatenate 'string  *hard-drive*
;                                ":logical-dir-compatibility.fasl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.fasl"))))
;(def-logical-directory "home;" *project-directory*)

#|

Summary of the changes to make to the mother/child input files
so that all CHILDES information is removed, and the words are in
a format which is acceptable to the lexicon files. Note the removal
is done in the order 1-8 shown below.

1. Characters which indicate lines to get rid of.
- a. [?]
- b. =?
- c. [+?
- d. @l
- e. @c
- f. xxx
- g. |

2. Words to remove from lines.
- a. oh, um, er

3. Characters to remove and remove the previous word from,
   unless the previous word is surrounded by angled brackets.
- a. [/] and [//]

4. Characters to remove, including characters between the signs.
- a. [ ]
- b. ( )
- c. < >

5. Characters to remove, including everything after the character.
- a. ,   ;; Added 18/11/98 GJ

6. Characters to change to other characters; note these are
   done in the order in which they appear in the list below.
- a. -'IS to 'S  e.g. there-'is -> there's
- b. -'US to 'S  e.g. let-'us -> let's
- c. -'NT to N'T  e.g. does-'nt -> doesn't
- d. -'HAS to 'S  e.g. there-'has -> there's
- e. -'HAVE to 'VE  e.g. what-'have -> what've
- f. P- to P  e.g. drop-ed -> dropped
- g. ERE-' to ERE'  e.g. here-'s -> here's
- h. -S to S  e.g. toe-s -> toes
- i. -'LL to 'LL  e.g. we-'ll -> we'll
- j. E- to [nothing]  e.g. give-ing -> giving
- k. ' to @  e.g. there's -> there@s

7. Characters to remove in a line.
- a. *MOT: *ADA: *CHI:
- b. +"   ;; Added 9/11/98 GJ
- c. - ? ! < > . : O  ;; Removed comma 18/11/98 GJ
- d. +...
- e. @f @o
- f. +,
- g. +/ +//
- h. /

8. Characters to remove and add a return instead.
- a. #

|#

;;---------------------- VARIABLES -----------------------------;;

;;; Set the project directory to the correct project folder.
;(setf *project-directory* "HD:Syntax project:Lexicon:")

(defvar output)
(defvar *child-code* nil)
(defvar *line-count* nil)
(defvar *report-comma-count* nil)
(defvar *comma-count* nil)
(defvar *filter-file-suffix* ".filt"
  "Suffix for the filtered input file.")
(defvar *comma-file-suffix* ".cm"
  "Suffix for the file containing the frequency of commas in the input.")

;;---------------------- FUNCTIONS -----------------------------;;

(defun filter-cfh ()
  (let ((temp nil) (input-file nil))
    (format t "~%Enter input directory (Default ~A): " *project-directory*)
    (setf temp (read-line))
    (if (> (length temp) 0)
      (progn
        (setf *project-directory* temp)
        (setf temp nil)))
    (format t "~%Enter input file: ")
    (setf input-file (read-line))
    (loop while (> (length input-file) 0) do
;      (format t "~%Enter code for child utterance if applicable: ")
;      (setf temp (read-line))
;      (if (> (length temp) 0)
;        (progn
;          (setf *child-code* temp)
;          (setf temp nil))
;        (setf *child-code* ""))
      (with-open-file (input (concatenate 'string *project-directory*
                                                  input-file)
                             :direction :input)
        (delete-if-exists (concatenate 'string *project-directory*
                                               input-file
                                               ".out"))
        (with-open-file (output (concatenate 'string *project-directory*
                                                     input-file
                                                     ".out")
                                :direction :output)
          (do ((a-line (read-line input nil 'eof nil)
                       (read-line input nil 'eof nil)))
              ((eq 'eof a-line) (print 'done))
            (when (keep-this-line-p a-line)
              (print-this-line (remove-all-superfluous-characters
                                 (remove-all-brackets
                                   (remove-all-repeats a-line)))
                               output)))))
      
      (format t "~%Enter input file: ")
      (setf input-file (read-line)))))


(defun filter ()
  (let ((temp nil) (input-file nil))
    (format t "~%Enter input directory (Default ~A): " *project-directory*)
    (setf temp (read-line))
    (if (> (length temp) 0)
      (progn
        (setf *project-directory* temp)
        (setf temp nil)))
    (format t "~%Enter input file (enter=quit): ")
    (setf input-file (read-line))
    (loop while (> (length input-file) 0) do
      (setf *line-count* 0)
      (setf *comma-count* 0)
;      (format t "~%Enter code for child utterance if applicable: ")
;      (setf temp (read-line))
;      (if (> (length temp) 0)
;        (progn
;          (setf *child-code* temp)
;          (setf temp nil))
;        (setf *child-code* ""))

      ;; GJ 4/12/98
      ;; Some utterances in the Childes file take up more than one
      ;; line - using read-line would read the utterance as two
      ;; separate utterances. So the return characters in the middle
      ;; of these utterances need to be removed. Whilst doing this,
      ;; we may as well remove tab characters as well.
      (remove-imbedded-return-chars (concatenate 'string *project-directory*
                                                 input-file)
                                    (concatenate 'string *project-directory*
                                                 input-file
                                                 ".1"))

      (with-open-file (input (concatenate 'string *project-directory*
                                                  input-file
                                                  ".1")
                           :direction :input)
        (delete-if-exists (concatenate 'string *project-directory*
                                               input-file
                                               *filter-file-suffix*))
        (with-open-file (output (concatenate 'string *project-directory*
                                                     input-file
                                                     *filter-file-suffix*)
                              :direction :output)
;          (princ "(" output ) (terpri output)
          (do ((a-line (read-line input nil 'eof nil)
                       (read-line input nil 'eof nil)))
              ((eq 'eof a-line) (print 'done))
            (setf *line-count* (+ *line-count* 1))
            (when (keep-this-line-p a-line)
              (print-this-line (change-quote-symbols
                                  (remove-all-superfluous-characters 
                                    (cater-for-verb-endings
                                      (remove-everything-after-char
                                        (remove-all-brackets
                                          (remove-all-repeats
                                            (remove-nonsense-words a-line)))))))
                               output)))))
;          (terpri output) (princ ")" output)

      (cond
        (*report-comma-count*
          (delete-if-exists (concatenate 'string *project-directory*
                                                 input-file
                                                 *comma-file-suffix*))
          (with-open-file (output (concatenate 'string *project-directory*
                                                       input-file
                                                       *comma-file-suffix*)
                                :direction :output)
            (format output "Comma frequency: ~A out of ~A lines. ~,2F percent."
              *comma-count* *line-count* (* (/ *comma-count* *line-count*) 100)))))

      ;; Delete the file created by remove-imbedded-return-chars.
      (delete-file (concatenate 'string *project-directory*
                                        input-file
                                        ".1"))
      (format t "~%Enter input file (enter=quit): ")
      (setf input-file (read-line)))))


(defun mass-filter ()
  "Do a mass filter on several files."
  (setf *project-directory* "HD:Syntax project:syntax:maternal-files:Raw mother files:")
  (dolist (input-file '("adam.mot" "bethan.mot" "conor.mot" "david.mot" "georgie.mot"
                        "jack.mot" "james.mot" "lucy.mot" "natalie.mot" "will.mot"))
    (setf *line-count* 0)
    (setf *comma-count* 0)

    (remove-imbedded-return-chars (concatenate 'string *project-directory*
                                               input-file)
                                  (concatenate 'string *project-directory*
                                               input-file
                                               ".1"))

    (with-open-file (input (concatenate 'string *project-directory*
                                                input-file
                                                ".1")
                         :direction :input)
      (delete-if-exists (concatenate 'string *project-directory*
                                             input-file
                                             *filter-file-suffix*))
      (with-open-file (output (concatenate 'string *project-directory*
                                                   input-file
                                                   *filter-file-suffix*)
                            :direction :output)
;        (princ "(" output ) (terpri output)
        (do ((a-line (read-line input nil 'eof nil)
                     (read-line input nil 'eof nil)))
            ((eq 'eof a-line) (print 'done))
          (setf *line-count* (+ *line-count* 1))
          (when (keep-this-line-p a-line)
            (print-this-line (change-quote-symbols
                                (remove-all-superfluous-characters 
                                  (cater-for-verb-endings
                                    (remove-everything-after-char
                                      (remove-all-brackets
                                        (remove-all-repeats
                                          (remove-nonsense-words a-line)))))))
                             output)))))
;        (terpri output) (princ ")" output)

    (cond
      (*report-comma-count*
        (delete-if-exists (concatenate 'string *project-directory*
                                                     input-file
                                                     *comma-file-suffix*))
        (with-open-file (output (concatenate 'string *project-directory*
                                                     input-file
                                                     *comma-file-suffix*)
                              :direction :output)
          (format output "Comma frequency: ~A out of ~A lines. ~,2F percent."
            *comma-count* *line-count* (* (/ *comma-count* *line-count*) 100)))))

    ;; Delete the file created by remove-imbedded-return-chars.
    (delete-file (concatenate 'string *project-directory*
                                      input-file
                                      ".1"))))

  
(defun remove-imbedded-return-chars (in-filename out-filename)
  "Some utterances in the Childes file take up more than one
   line - using read-line would read the utterance as two
   separate utterances. This function removes the return
   characters that are in the middle of these utterances."
  (let ((prev-char nil))
    (with-open-file (input in-filename :direction :input)
      (with-open-file (output out-filename :direction :output
                                           :if-exists :overwrite)
        (do ((a-char (read-char input nil 'eof nil)
                     (read-char input nil 'eof nil)))
            ((eq 'eof a-char) t)
          (cond ((and (equal a-char #\*) (equal prev-char #\Return))
                  (terpri output)
                  (princ a-char output))
                ((equal a-char #\Tab)
                  (princ *whitespace* output))
                ((not (equal a-char #\Return))
                  (princ a-char output))
                (t nil))
          (setf prev-char a-char))
        (terpri output)))))
 

(defun keep-this-line-p (a-line)
  (unless 
    (or (search "[?]" a-line)
        (search "=?" a-line)
        (search "[+?" a-line)
        (search "@l" a-line)
        (search "@c" a-line)
        (search "xxx" a-line)
        (search "|" a-line))  
    t))


(defun remove-nonsense-words (a-line)
  "Remove any words that are not required."
  (list-to-string
    (remove "er"
      (remove "um"
        (remove "oh" (coerce-string-into-list a-line)
          :test #'equal)
        :test #'equal)
      :test #'equal)))


(defun remove-all-repeats (a-line)
  "A [/] code means that a word has been repeated - the word
   will be immediately before the [/] code. If a line contains
   a [/] code, then remove the word prior to the code."
  (let ((list-string (coerce-string-into-list a-line))
        (n1 nil) (n2 nil) (prev-word nil))
;(format t "~%~A" a-line)
    (setf n1 (search (list "[/]") list-string :test #'equal))
    (setf n2 (search (list "[//]") list-string :test #'equal))
;(format t "~%~A ~A" n1 n2)
    (loop while (or n1 n2) do
      (cond
        (n1
          ;; Check the word prior to the square bracket.
          ;; If its a pause or contains an angled bracket, leave it.
          ;; Otherwise remove the prior word.
          (setf prev-word (nth (- n1 1) list-string))
          (cond
            ((equal "#" prev-word)
              ;; Remove word before the pause.
              (setf list-string (append (subseq list-string 0 (- n1 2))
                                        (subseq list-string (- n1 1) n1)
                                        (subseq list-string (+ n1 1)
                                                            (length list-string)))))
            ((or (member "<" (convert-word-to-letter-list prev-word)
                         :test #'equal)
                 (member ">" (convert-word-to-letter-list prev-word)
                         :test #'equal))
              (setf list-string (append (subseq list-string 0 n1)
                                        (subseq list-string (+ n1 1)
                                                            (length list-string)))))
            (t ;; Remove previous word.
              (setf list-string (append (subseq list-string 0 (- n1 1))
                                        (subseq list-string (+ n1 1)
                                                            (length list-string)))))))

        (n2
          ;; Check the word prior to the square bracket.
          ;; If its a pause or contains an angled bracket, leave it.
          ;; Otherwise remove the prior word.
          (setf prev-word (nth (- n2 1) list-string))
          (cond
            ((equal "#" prev-word)
              ;; Remove word before the pause.
              (setf list-string (append (subseq list-string 0 (- n2 2))
                                        (subseq list-string (- n2 1) n2)
                                        (subseq list-string (+ n2 1)
                                                            (length list-string)))))
            ((or (member "<" (convert-word-to-letter-list prev-word)
                         :test #'equal)
                 (member ">" (convert-word-to-letter-list prev-word)
                         :test #'equal))
              (setf list-string (append (subseq list-string 0 n2)
                                        (subseq list-string (+ n2 1)
                                                            (length list-string)))))
            (t ;; Remove previous word.
              (setf list-string (append (subseq list-string 0 (- n2 1))
                                        (subseq list-string (+ n2 1)
                                                            (length list-string))))))))

      (setf n1 (search (list "[/]") list-string :test #'equal))
      (setf n2 (search (list "[//]") list-string :test #'equal)))

    (list-to-string list-string)))


(defun remove-all-brackets (a-line)
  (remove-brackets-and-string-between "[" "]"
   (remove-brackets-and-string-between "[" "]"
    (remove-brackets-and-string-between "(" ")"
     (remove-brackets-and-string-between "(" ")"
      (remove-brackets-and-string-between "<" ">" a-line))))))
;; some duplicated to take care of repetition


(defun remove-everything-after-char (a-line)
  "Remove everything after specific characters in the line."
  (cond
    ((search "," a-line)
      (setf *comma-count* (+ *comma-count* 1))
      (subseq a-line 0 (search "," a-line)))
    (t a-line)))


(defun cater-for-verb-endings (a-line)
  "Change the verb ending for any verbs in
   the sentence so that they and up being output correctly.
   e.g. give-ing wants to be giving, does-'nt wants to be doesn't."
  ;; Ensure all upper case characters before beginning so that
  ;; character comparisons are correct.
  (setf a-line (string-upcase a-line))
  (let ((position nil))

    ;; Change -'is to 's; e.g. there-'is -> there's.
    (setf position (search "-'IS" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                "'S"
                                (subseq a-line (+ position 4)
                                               (length a-line))))
      (setf position (search "-'IS" a-line)))

    ;; Change -'us to 's; e.g. let-'us -> let's.
    (setf position (search "-'US" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                "'S"
                                (subseq a-line (+ position 4)
                                               (length a-line))))
      (setf position (search "-'US" a-line)))

    ;; Change -'nt to n't; e.g. does-'nt -> doesn't.
    (setf position (search "-'NT" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                "N'T"
                                (subseq a-line (+ position 4)
                                               (length a-line))))
      (setf position (search "-'NT" a-line)))

    ;; Change -'has to 's; e.g. there-'has -> there's.
    (setf position (search "-'HAS" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                "'S"
                                (subseq a-line (+ position 5)
                                               (length a-line))))
      (setf position (search "-'HAS" a-line)))

    ;; Change -'have to 've; e.g. what-'have -> what've.
    (setf position (search "-'HAVE" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                "'VE"
                                (subseq a-line (+ position 6)
                                               (length a-line))))
      (setf position (search "-'HAVE" a-line)))

    ;; For past tense p- add a p.
    ;; e.g. drop-ed -> dropped. 
    (setf position (search "P-" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               (+ position 1))
                                "P"
                                (subseq a-line (+ position 2)
                                               (length a-line))))
      (setf position (search "P-" a-line)))

    ;; Change any ere-'s to ere's before removing
    ;; e- substrings.
    ;; e.g. here-'s -> here's 
    ;; This is required because otherwise some words e.g. here, there,
    ;; don't want the e removed.
    (setf position (search "ERE-'" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                "ERE'"
                                (subseq a-line (+ position 5)
                                               (length a-line))))
      (setf position (search "ERE-'" a-line)))

    ;; For plurals, take out the -.
    ;; e.g. toe-s -> toes. Have to do this here and
    ;; not in superfluous-characters because it would get
    ;; picked up by the next bit of code and be set to "tos".
    (setf position (search "-S" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                (subseq a-line (+ position 1)
                                               (length a-line))))
      (setf position (search "-S" a-line)))

    ;; For -'ll, take out the -.
    ;; e.g. we-'ll -> we'll. Have to do this here and
    ;; not in superfluous-characters because it would get
    ;; picked up by the next bit of code and be set to "w'll".
    (setf position (search "-'LL" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                (subseq a-line (+ position 1)
                                               (length a-line))))
      (setf position (search "-'LL" a-line)))

    ;; Remove any e- substrings.
    ;; e.g. give-ing -> giving.
    (setf position (search "E-" a-line))
    (loop while (and position
                     (not (= position 0))) do
      (setf a-line (concatenate 'string
                                (subseq a-line 0 
                                               position)
                                (subseq a-line (+ position 2)
                                               (length a-line))))
      (setf position (search "E-" a-line))))
  a-line)


(defun remove-all-superfluous-characters (a-line)
  (remove-all-substrings '("*MOT:" "*ADA:" "*CHI:"
                           "+\"" ;; Added 9/11/98 GJ
                           "-" "?" "!" "+..." "<" ">" "@f"  
                           "@o"  "+," "," "0"
                           "." ":" "+/" "+//" "/")
                         a-line))


(defun treat-pause-separate-utterances (a-line output)
  "Treat a pause as two separate utterances, one occurring
   before the pause and one occurring after it."
  (dotimes (n (length a-line))
    (if (char= (aref a-line n) #\#)
      (terpri output)
      (princ (aref a-line n) output)))
  (terpri output))


;(defun remove-all-superfluous-characters-cfh (a-line)
;  (remove-all-substrings '("*MOT:" "*ADA:"
;                           "+..." "<" ">" "+" "+" "+" "@f"  
;                           "@o" "@o" "@o"  "+," "," "'")
;                         a-line))


(defun remove-all-substrings (sub-string-list main-string)
  (dolist (x sub-string-list)
    (setf main-string (remove-sub-string x main-string)))
  main-string)


(defun remove-sub-string (sub-string main-string)
  "Remove every occurrence of a substring within a string of text."
  (let ((position nil))
    (setf position (search sub-string main-string))
    (loop while position do
      (setf main-string (concatenate 'string
                          (subseq main-string 0 position)
                          (subseq main-string (+ position (length sub-string))
                                              (length main-string))))
      (setf position (search sub-string main-string))))
  main-string)


(defun remove-brackets-and-string-between (bracket-open bracket-close
                                                        main-string)
  (let ((open-pos  (search bracket-open main-string))
        (close-pos (search bracket-close main-string)))
    (if (and open-pos close-pos)
      (concatenate 'string
                   (subseq main-string 0 open-pos)
                   " "
                   (subseq main-string (+ close-pos 1)))
      main-string)))


;(defun print-this-line-cfh (a-line output)
;  "Print the line treating pauses as the same utterance."
;  (cond ((search "#" a-line) (treat-pause-same-utterance a-line output))
;        (t (princ a-line output)
;           (terpri output))))


(defun print-this-line (a-line output)
  "Print the line treating pauses as different utterances."
  (cond ((search "#" a-line) (treat-pause-separate-utterances a-line output))
        (t
          (princ a-line output)
          (terpri output))))


(defun treat-pause-same-utterance (a-line output)
  "Treat a pause as being part of the same utterance."
  (dotimes (n (length a-line))
    (if (not (char= (aref a-line n) #\#))
      (princ (aref a-line n) output)))
  (terpri output))