;; F.R. Gobet & G. Jones, October 1998.
;; ------------------------------------
;; A set of functions to process a word/phoneme lexicon file.
;; The file is taken from http://www.speech.cs.cmu.edu/cgi-bin/cmudict
;; and consists of a text list of lines. Each line consists of
;; an orthographic representation of a word followed by a list
;; of the phonemes which represent the pronunciation of the word.
;; The file has a header and footer that can be ignored, and also
;; contains improper termination characters, and so the file is
;; recoded before processing. 
;; Accessing the file is done by means of a hash table. Each entry in
;; the hash table consists of two items for each word in the lexicon:
;; the orthographic representation of the word, and the phonemic
;; representation of the word. The two items are added to the hash table
;; one after the other. IT IS IMPORTANT WHICH ITEM IS ADDED FIRST
;; because the first item provides quick accessed, but the second
;; will be slower. Therefore if most accessing will involve inputting
;; the orthographic representation to get the phonemic, then add the
;; orthographic representation first (and vice versa for primarily
;; phonemic access). 
;; The functions to access the hash table enable access of the phonemic
;; representation from the orthographic representation, and the
;; orthographic representation from the phonemic representation.
;; Access can also be achieved for sentences or a long list of phonemes.
;; NOTE: Using the hash table means no partial matching will be difficult
;; to do. Partial matching is NOT currently implemented.
;;
;; CHANGE RECORD
;; Ini  Date      Change summary & functions added, deleted, or altered.
;; ---  --------  ------------------------------------------------------
;; GJ   16/10/98  Initial program.
;;                ------------------------------------------------------
;; GJ   30/10/98  Took out the general functions section and put it in
;;                a general-function.lisp file so that this file can be
;;                loaded by any lisp program. This meant some of the
;;                constants moved too.
;;                ------------------------------------------------------
;; GJ   02/11/98  Ensure that in the convert function, the string
;;                to access the hash table with is in upper case
;;                as hash table entries are in upper case.
;;                ------------------------------------------------------
;; GJ   27/02/99  Added a check of lisp-implementation-version to determine
;;                whether to load fasl files or pfsl (vsn 4+) files.
;;                ------------------------------------------------------
;; GJ   03/03/99  Ensure that the item in convert is upper case.
;;                ------------------------------------------------------
;; GJ   03/03/99  All lexicon related files will be loaded from
;;                lexicon-loader.lisp now, so the loads for logical-dir and 
;;                general-functions can be commented out. Also
;;                *project-directory* should be used for filepaths.
;;                ------------------------------------------------------
;; GJ   11/03/99  Before writing to output files, delete the file if it
;;                already exists. This is because overwrite in Lisp
;;                leaves the tail end of the previous file if the
;;                overwritten information is shorter.
;; ======== END OF CHANGE RECORD ========


;;---------------------- CONSTANTS -----------------------------;;

(defconstant *hash-table-size* 225000 "Size of the hash table. The size is 2
             for each entry in the table, 1 for the orthographic rep. and 1 for
             the phonemic rep. The size is therefore the number of items in
             the lexicon plus a few thousand for any additions." )


;;---------------------- VARIABLES -----------------------------;;

(defvar *hash-table* nil "Hash table for the words and phonemes.")


;;---------------------- FUNCTIONS -----------------------------;;

;;; ============================================================
;;; Load in any files required by functions in this file.
;;; ============================================================

;(cond
;  ((member "4" (mapcar #'string (coerce (lisp-implementation-version) 'list))
;           :test #'equal)
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.pfsl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.pfsl")))
;  (t
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.fasl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.fasl"))))
;(def-logical-directory "home;" *project-directory*)

;;; ============================================================
;;; Functions to recode the lexicon into a file which is
;;; easily readable by Lisp functions.
;;; ============================================================


(defun recode-lexicon ()
  "Recode the word/phoneme lexicon so that the
   line termination characters can be read in Lisp,
   and the only lines that are in the file are 
   words and phonemes."
  (let ((skip-to-end-of-line nil)
        (first-char-of-line t))
    (delete-if-exists *recoded-lexicon-filepath*)
    (with-open-file (output *recoded-lexicon-filepath*
                            :direction :output)
      (with-open-file (input *raw-lexicon-filepath*
                             :direction :input)
        (do ((ch (read-char input nil 'eof)
                 (read-char input nil 'eof)))
            ((eq 'eof ch))
          (cond 
           ;; If the first char is not alphabetic, then
           ;; skip every subsequent character until the next
           ;; line.
           ((and first-char-of-line
                 (not (alphabetic-character ch)))
            ;		    (print ch)
            ;		    (print "skip to end")
            (setf first-char-of-line nil)
            (setf skip-to-end-of-line t))
           
           ;; If we're not skipping characters and we're not
           ;; at the end of the line, output the char to file.
           ((and (not skip-to-end-of-line)
                 (not (equalp ch #\Linefeed)))
            ;              (print "write char")
              
        
            (setf first-char-of-line nil)
             (princ ch output))
           
           ;; If we're not skipping characters and we're 
           ;; at the end of the line, put a proper termination
           ;; character at the end.
           ((and (not skip-to-end-of-line)
                 (equalp ch #\Linefeed))
            ;              (print "write terminal")
            (terpri output)
            (setf first-char-of-line t))
           
           ;; If characters are being skipped and the end of
           ;; the line has been reached, reset the skip flag.
           ((and skip-to-end-of-line
                 (equalp ch #\Linefeed))
            ;              (print "dont skip any more")
            (setf skip-to-end-of-line nil)
            (setf first-char-of-line t))
           
           (t nil)))))))


;;; ============================================================
;;; Functions to create the hash table.
;;; ============================================================

(defun init-hash-table ()
  "Create the hash-table. Note a rehash-threshold of 1 indicates the table 
   must be completely full before it is expanded."
  (setf *hash-table* (make-hash-table :size *hash-table-size* :test #'equal
                                      :rehash-threshold 1)))


(defun create-lexicon ()
  "Create a hash-table for the word/phoneme lexicon. The
   table either creates the first key as the orthographic
   rep. or the phonemic rep. depending on *primary-access-orthographic*."
  (init-hash-table)
  (with-open-file (input *recoded-lexicon-filepath*
                         :direction :input)
    (do ((n 0)
         (a-line (read-line input nil 'eof)
                 (read-line input nil 'eof))
         (orth-word nil))
        ((equalp 'eof a-line))
      ;; Before processing the input, there are various symbols that
      ;; the orthographic word could contain which are problematic:
      ;; 1. Parentheses. This indicates the same orthographic word having
      ;;    more than one phonemic representation. The duplicate words are ignored.
      ;; 2. Full stops (periods) and numerics. These lines are ignored.
      ;; 3. Quotes. Quotes are difficult to handle in Lisp, so the quote
      ;;    characters are changed to @ signs.
      (setf orth-word (get-first-word a-line))
      (if (and (not (string-contains-parentheses-p orth-word))
               (not (string-contains-full-stop-p orth-word))
               (not (string-contains-numerics-p orth-word)))
        (let* ((recoded-input (if (string-contains-quote-p orth-word)
                                (coerce-string-into-list-of-two-lists
                                  (change-quote-symbols a-line))
                                (coerce-string-into-list-of-two-lists a-line)))
               (word (string (first recoded-input)))
               (pron (second recoded-input)))
          (incf n)
          (when (= (mod n 1000) 0)
            (format t "~A " n))
          (cond
            (*primary-access-orthographic*
              (setf (gethash word *hash-table*) pron)
              (setf (gethash pron *hash-table*) word))
            (t
              (setf (gethash pron *hash-table*) word)
              (setf (gethash word *hash-table*) pron))))))))


;;; ============================================================
;;; Functions to access the hash table.
;;; ============================================================

(defun convert (item)
  "Find the corresponding phonemic representation if an orthographic
   word is input, or the corresponding orthographic word if a list
   of phonemes is input."
  (cond
    ((stringp item)
      (gethash (string-upcase item) *hash-table*))
    ((listp item)
      (setf item (coerce-string-into-list
                   (list-to-string item)))
      (gethash item *hash-table*))
    (t nil)))


(defun convert-sequence (item-list)
  "Find the corresponding phonemic or orthographic representations
   of a list of items (i.e. an input sentence of words or phonemes)."
  (let ((item '()) (conversion '()))
    (cond 
      ((stringp item-list)  ;; Orth to phon
        (setf item-list (coerce-string-into-list item-list))
        (dolist (item item-list)
          (setf conversion (append conversion (convert (string item))))))
      (t                    ;; Phon to orth
        ;; NB Phon to orth may not be successful because the first
        ;; match fitting a list of phonemes is used. For example, the
        ;; initial phonemes for ASK and ASKED are the same, so ASK will
        ;; always be taken as the word, and not ASKED.
        (loop while item-list do
          (setf item (append item (list (car item-list))))
          (let ((orth-conversion (convert item)))
            (if orth-conversion
              (progn
                (setf conversion (append conversion (list orth-conversion)))
                (setf item nil)))
            (setf item-list (cdr item-list))))))
    (format t "~%~S " conversion)))


(defun test ()
  "Read in a file test-list which is a set of test
   items (words and phonemes) for the hash table."
  (setf *recoded-lexicon-filepath* "HD:Lexicon:test-list")
  (init-hash-table)
  (create-lexicon))