;; G. Jones, February 1999.
;; -----------------------
;; A set of functions to read in the MRC Psycholinguistics database
;; and extract specific information from it. The database can be
;; obtained from http://wapsy.psy.uwa.edu.au/mrc2.html.
;;
;; CHANGE RECORD
;; Ini  Date      Change summary & functions added, deleted, or altered.
;; ---  --------  ------------------------------------------------------
;; GJ   12/02/99  Initial program.
;; ---  --------  ------------------------------------------------------
;; GJ   02/03/99  Changed create-word-details so that the word can contain
;;                quotes, which will be changed to a new quote symbol as
;;                standard quotes are difficult to handle in lisp.
;;                ------------------------------------------------------
;; GJ   03/03/99  All lexicon related files will be loaded from
;;                lexicon-loader.lisp now, so the loads for logical-dir and 
;;                general-functions can be commented out.
;;                ------------------------------------------------------
;; GJ   03/03/99  When a word has more than one category (denoted by the
;;                word having more than one entry), put the attributes
;;                as lists within lists in the hash table. This meant
;;                changing create-word-details and get-attributes.
;;                ------------------------------------------------------
;; ======== END OF CHANGE RECORD ========


;;---------------------- CONSTANTS -----------------------------;;

(defconstant *psycho-db-size* 115000 "Size of the psycholinguistics
             database hash table. The size is 2
             for each entry in the table, 1 for the word and 1 for
             the lists of attributes for the word (because some words
             can exist as more than one category e.g. HARASS, each set
             of attributes are listed). The size is therefore
             the number of items in the lexicon plus a few thousand
             for any additions." )


;;---------------------- VARIABLES -----------------------------;;

(defvar *psycho-db* nil "Hash table for the words and attributes.")
(defvar *psycho-db-filepath* (concatenate 'string
                                           *project-directory*
                                           "mrc2.dct"))
(defvar *word-begin-column* 52)
(defvar *word-termination-character* "|")
(defvar *attributes-list* '((5 5 N   "Number of syllables")
                            (6 10 N  "Kucera and Francis word frequency (range 0-69971)")
                            (29 31 N "Concreteness (range 100-700)")
                            (35 37 N "Mean Colerado Meaningfulness*100 (range 100-700)")
                            (38 40 N "Mean Pavio Meaningfulness*100 (range 100-700)")
                            (41 43 N "Age of Acquisition*100; (range 100-700)")
                            (45 45 A 
            "Noun,adJ,Verb,Adve,pRep,Conj,pronoUn,Interjection,Pastptcpl,Other")
                            (46 46 A "PDWType Noun,Verb,adJ,Other"))
  "A list of the columns to extract from the database, whether
   the column is numeric (N) or not (A), and a description of the column.")
(defvar *alphabetics-only* t
  "Boolean value stating whether to accept only words containing
   only alphabetic characters, or to accept all words.")


;;---------------------- FUNCTIONS -----------------------------;;

;;; ============================================================
;;; Load in any files required by functions in this file.
;;; ============================================================

;(cond
;  ((member "4" (mapcar #'string (coerce (lisp-implementation-version) 'list))
;           :test #'equal)
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.pfsl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.pfsl")))
;  (t
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.fasl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.fasl"))))

;(def-logical-directory "home;" *project-directory*)


;;; ============================================================
;;; Functions to create the hash table.
;;; ============================================================

(defun init-psycho-db ()
  "Create the hash-table. Note a rehash-threshold of 1 indicates the table 
   must be completely full before it is expanded."
  (setf *psycho-db* (make-hash-table :size *psycho-db-size* :test #'equal
                                     :rehash-threshold 1)))


(defun create-word-details ()
  "Create a psycholinguistics database hash-table for the
   word details. The table will have the word as its key."
  (init-psycho-db)
  (with-open-file (input *psycho-db-filepath*
                         :direction :input)
    (do* ((att-list '() '())
          (attribute nil nil)
          (a-line (read-line input nil 'eof)
                  (read-line input nil 'eof))
          (word (list-to-string-no-spaces
                  (substitute *new-quote-symbol* *quote*
                              (coerce-chars-in-string-into-list
                                (retrieve-db-word-from-string a-line))
                                :test #'equal))
                (list-to-string-no-spaces
                  (substitute *new-quote-symbol* *quote*
                              (coerce-chars-in-string-into-list
                                (retrieve-db-word-from-string a-line))
                                :test #'equal))))
        ((equalp 'eof a-line))
;(format t "~%~A ~A" word (list-to-string-no-spaces (remove *new-quote-symbol*
;                                                           (coerce-chars-in-string-into-list word)
;                                                           :test #'equal)))
      (cond
        ((or (and *alphabetics-only*
                  (string-is-all-alphabetics-p 
                    (list-to-string-no-spaces (remove *new-quote-symbol*
                                                      (coerce-chars-in-string-into-list word)
                                                      :test #'equal))))
             (not *alphabetics-only*))
          (dolist (attribute-columns *attributes-list*)
            (cond
              ((> (length a-line) (second attribute-columns))
                (setf attribute (return-string-between-columns a-line
                                                               (first attribute-columns)
                                                               (second attribute-columns)))
                (cond
                  ((equal (string (third attribute-columns)) "N")
                    (setf att-list (append att-list (list
                                     (truncate (calculate-number-value-from-string attribute))))))
                  (t
                    (setf att-list (append att-list (list attribute))))))))
          ;; Word may already exist in the hash table if it can be more than
          ;; one category (in which case another list of attributes are added).
          (cond
            ((gethash (string-upcase word) *psycho-db*)
              (setf (gethash (string-upcase word) *psycho-db*)
                (append (gethash (string-upcase word) *psycho-db*) (list att-list))))
            (t
              (setf (gethash (string-upcase word) *psycho-db*) (list att-list)))))))))


(defun retrieve-db-word-from-string (a-string)
  "Return the word in the database from the string given."
  (cond
    ((and (stringp a-string)
          (> *word-begin-column* 0)
          (>= (length a-string) *word-begin-column*))
      ;; The column that the word begins on assumes that the
      ;; first column begins at zero.
      (let ((string-list (coerce-chars-in-string-into-list a-string))
            (index (- *word-begin-column* 1)) (word ""))
        (loop while (and (<= index (length a-string))
                         (not (equal (nth index string-list)
                                     *word-termination-character*))) do
          (setf word (format nil "~A~A" word (nth index string-list)))
          (incf index))
        word))
    (t "")))


;;; ============================================================
;;; Functions to access the hash table.
;;; ============================================================

(defun get-attributes (word)
  "Find the attributes for a word. Some words will have more than one set."
  (setf word (string-upcase word))
  (let ((word-attributes (gethash word *psycho-db*)) (n 0) (n-word-att 1))
    (dolist (attributes word-attributes)
      (cond
        ((> (length word-attributes) 1)
          (format t "~%Word: ~A(~A)~%" word n-word-att)
          (incf n-word-att))
        (t 
          (format t "~%Word: ~A~%" word)))
      (setf n 0)
      (dolist (attribute-desc *attributes-list*)
        (format t "~A: ~A~%" (fourth attribute-desc) (nth n attributes))
        (incf n)))))